#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SDL.h"
#include "sdlxpm.h"
#include "zero.xpm"
#include "one.xpm"
#include "two.xpm"
#include "three.xpm"
#include "four.xpm"
#include "five.xpm"
#include "six.xpm"
#include "seven.xpm"
#include "eight.xpm"
#include "nine.xpm"

/* ^U */
#define FALSE	0
#define TRUE	1

/* _ */
struct Point {
	int no;
	int x;
	int y;
	int z;
};

/*  */
struct Face {
	int n;
	int p[8];
};

/*  */
struct Line {
	int n;
	int p[8];
};

/*  */
struct Side {
	int p[2];
};

/* _ */
struct Point *point;

/*  */
struct Face *face;

/*  */
struct Line *line;

/*  */
struct Side *side;

/*  */
static SDL_Color white = { 0xff, 0xff, 0xff, 0xff };

/* DF */
static SDL_Color gray = { 0x20, 0x20, 0x20, 0x20 };

/*  */
static SDL_Surface *number[10];

/*
	_`
*/
static void drawPoint(SDL_Surface *s, int x, int y, SDL_Color c)
{
	Uint32 pix;

	if(x < 0 || y < 0 || x >= s->w || y >= s->h)
		return;

	if(SDL_MUSTLOCK(s))
		SDL_LockSurface(s);

	pix = SDL_MapRGB(s->format, c.r, c.g, c.b);

	if(s->format->BytesPerPixel == 1) {
		Uint8 *p = &((Uint8 *)s->pixels)[s->pitch * y + x * s->format->BytesPerPixel];
		*(Uint8 *)p = pix;
	} else if(s->format->BytesPerPixel == 2) {
		Uint8 *p = &((Uint8 *)s->pixels)[s->pitch * y + x * s->format->BytesPerPixel];
		*(Uint16 *)p = pix;
	} else if(s->format->BytesPerPixel == 4) {
		Uint8 *p = &((Uint8 *)s->pixels)[s->pitch * y + x * s->format->BytesPerPixel];
		*(Uint32 *)p = pix;
	}

	if(SDL_MUSTLOCK(s))
		SDL_UnlockSurface(s);
}

/*
	ϐ̒l
*/
static void swap(int *x, int *y)
{
	int tmp = *x;
	*x = *y;
	*y = tmp;
}

/*
	`
*/
static void drawLine(SDL_Surface *s, SDL_Rect *clip, int x0, int y0, int x1, int y1, SDL_Color c)
{
	if(x0 > x1) {
		swap(&x0, &x1);
		swap(&y0, &y1);
	}

	if(x0 == x1) {
		int y;

		if(y0 > y1)
			swap(&y0, &y1);

		for(y = y0; y <= y1; y++)
			drawPoint(s, clip->x + x0, clip->y + y, c);
	} else if(y0 == y1) {
		int x;

		for(x = x0; x <= x1; x++)
			drawPoint(s, clip->x + x, clip->y + y0, c);
	} else if(x1 - x0 > abs(y1 - y0)) {
		double y, dy = (double )(y1 - y0) / (x1 - x0);
		int x;

		for(x = x0, y = y0 + .5; x <= x1; x++, y += dy)
			drawPoint(s, clip->x + x, clip->y + y, c);
	} else {
		double x, dx;
		int y;

		if(y0 > y1) {
			swap(&x0, &x1);
			swap(&y0, &y1);
		}
		dx = (double )(x1 - x0) / (y1 - y0);

		for(y = y0, x = x0 + .5; y <= y1; y++, x += dx)
			drawPoint(s, clip->x + x, clip->y + y, c);
	}
}

static void drawCircle(SDL_Surface *s, int x0, int y0, int r, SDL_Color c)
{
	int x = r, y = 0, d = -2 * r + 3;

	while(x >= y) {
		printf("x=%d, y=%d, d=%d\n", x, y, d);
		fflush(stdout);

		drawPoint(s, x0 + x, y0 + y, c);
		drawPoint(s, x0 - x, y0 + y, c);
		drawPoint(s, x0 + x, y0 - y, c);
		drawPoint(s, x0 - x, y0 - y, c);
		drawPoint(s, x0 + y, y0 + x, c);
		drawPoint(s, x0 - y, y0 + x, c);
		drawPoint(s, x0 + y, y0 - x, c);
		drawPoint(s, x0 - y, y0 - x, c);

		if(d >= 0) {
			x--;
			d -= 4 * x;
		}
		y++;
		d += 4 * y + 2;
	}
}

/*
	l\
*/
static void drawNumber(SDL_Surface *screen, SDL_Rect *clip, int x, int y, int n)
{
	SDL_Rect dst = { 0, 0, 6, 8 };
	char buf[8], *p;

	sprintf(buf, "%d", n);

	if(y >= clip->h / 2)
		dst.y = clip->y + y + 2;
	else
		dst.y = clip->y + y - 8;

	if(x >= clip->w / 2)
		dst.x = clip->x + x + 2;
	else
		dst.x = clip->x + x - strlen(buf) * 6;

	for(p = buf; *p != 0; p++, dst.x += 6)
		SDL_BlitSurface(number[*p - '0'], NULL, screen, &dst);
}

/*
	CSV̒l𓾂
*/
static int getCsvValue(char **p, char **value, int *length)
{
	if(*p == NULL) {
		*value = NULL;
		*length = 0;
		return FALSE;
	}

	while(**p == ' ' || **p == '\t')
		(*p)++;

	if(**p == 0 || **p == '\r' || **p == '\n' || **p == ';') {
		*value = NULL;
		return FALSE;
	}

	*value = *p;

	if(**p != ',')
		while(**p != 0 && **p != '\r' && **p != '\n' && **p != ' ' && **p != '\t' && **p != ';' && **p != ',')
			(*p)++;

	*length = (int)(*p - *value);
	if(**p == ',')
		(*p)++;
	return TRUE;
}

/*
	R}h𓾂
*/
static char getCommand(char **p)
{
	char *v;
	int len;

	getCsvValue(p, &v, &len);

	if(v == NULL)
		return 0;
	else
		return toupper((int )*v);
}

/*
	𐮐ɕϊ
*/
static int toInt(const char *str, int len)
{
	char buf[16], *q;
	const char *p;

	for(p = str, q = buf; /*isdigit((int )*p) &&*/ q < buf + sizeof(buf) && (int)(p - str) < len; p++, q++)
		*q = *p;
	*q = 0;

	return atoi(buf);
}

/*
	l𓾂
*/
static int getValue(char **p)
{
	char *v;
	int len;

	getCsvValue(p, &v, &len);

	if(v == NULL)
		return INT_MIN;
	else
		return toInt(v, len);
}

/*
	_̐߂
*/
static int getPointCount(void)
{
	int count;

	if(point == NULL)
		return 0;

	for(count = 0; point[count].no >= 0; count++)
		;
	return count;
}

/*
	_ǉ
*/
static struct Point *addPoint(void)
{
	int count = getPointCount();

	if(point == NULL)
		point = malloc(sizeof(struct Point) * 2);
	else
		point = realloc(point, sizeof(struct Point) * (count + 2));

	point[count + 0].no = 0;
	point[count + 1].no = -1;
	return &point[count];
}

/*
	ʂ̐߂
*/
static int getFaceCount(void)
{
	int count;

	if(face == NULL)
		return 0;

	for(count = 0; face[count].n >= 0; count++)
		;
	return count;
}

/*
	ʂǉ
*/
static struct Face *addFace(void)
{
	int count = getFaceCount();

	if(face == NULL)
		face = malloc(sizeof(struct Face) * 2);
	else
		face = realloc(face, sizeof(struct Face) * (count + 2));

	face[count + 0].n = 0;
	face[count + 1].n = -1;
	return &face[count];
}

/*
	̐߂
*/
static int getLineCount(void)
{
	int count;

	if(line == NULL)
		return 0;

	for(count = 0; line[count].n >= 0; count++)
		;
	return count;
}

/*
	ǉ
*/
static struct Line *addLine(void)
{
	int count = getLineCount();

	if(line == NULL)
		line = malloc(sizeof(struct Line) * 2);
	else
		line = realloc(line, sizeof(struct Line) * (count + 2));

	line[count + 0].n = 0;
	line[count + 1].n = -1;
	return &line[count];
}

static int getLineSum(void)
{
	struct Line *l;
	int sum = 0;

	if(line == NULL)
		return 0;

	for(l = line; l->n >= 0; l++)
		sum += l->n - 1;
	return sum;
}

/*
	ӂ̐߂
*/
static int getSideCount(void)
{
	int count;

	if(side == NULL)
		return 0;

	for(count = 0; side[count].p[0] >= 0; count++)
		;
	return count;
}

/*
	ӂǉ
*/
static struct Side *addSide(void)
{
	int count = getSideCount();

	if(side == NULL)
		side = malloc(sizeof(struct Side) * 2);
	else
		side = realloc(side, sizeof(struct Side) * (count + 2));

	side[count + 0].p[0] = 0;
	side[count + 1].p[0] = -1;
	return &side[count];
}

/*
	ӂ
*/
static struct Side *findSide(int p0, int p1)
{
	struct Side *s;

	if(side == NULL)
		return NULL;

	for(s = side; s->p[0] >= 0; s++)
		if((s->p[0] == p0 && s->p[1] == p1) || (s->p[0] == p1 && s->p[1] == p0))
			return s;
	return NULL;
}

/*
	ԍ璸_߂
*/
static struct Point *findPoint(int no)
{
	struct Point *p;

	for(p = point; p->no >= 0; p++)
		if(p->no == no)
			return p;
	return NULL;
}

static int getPointIndex(int no)
{
	struct Point *p = findPoint(no);

	if(p == NULL)
		return -1;
	else
		return (int)(p - point);
}

/*
	a߂
*/
static int getRadius(void)
{
	struct Point *p;
	int r = 0;

	for(p = point; p->no >= 0; p++)
		if(r * r < p->x * p->x + p->y * p->y + p->z * p->z)
			r = p->x * p->x + p->y * p->y + p->z * p->z;
	return sqrt(r);
}

/*
	pXgqt@C𓾂
*/
static char *getFileBaseName(char *base, const char *path)
{
	const char *p, *start = path, *end;

	for(p = path; *p != 0; p++)
		if(*p == '/' || *p == '\\')
			start = p + 1;

	for(p = start; *p != 0 && *p != '.'; p++)
		;
	end = p;

	sprintf(base, "%.*s", (int)(end - start), start);
	return base;
}

/*
	pXfBNg𓾂
*/
static char *getDirName(char *dir, const char *path)
{
	const char *p, *end = NULL;

	for(p = path; *p != 0; p++)
		if(*p == '\\' || *p == '/')
			end = p;

	if(end == NULL)
		strcpy(dir, "");
	else
		sprintf(dir, "%.*s", (int)(end - path) + 1, path);
	return dir;
}

/*
	ft@Cǂݍ
*/
static int readModelFile(const char *path)
{
	struct Point *new_point;
	struct Face *new_face, *f;
	struct Line *new_line;
	FILE *fp;
	int i, j, row = 0;
	char buf[256], *p;

	if(path == NULL || strcmp(path, "") == 0)
		return FALSE;
	if((fp = fopen(path, "rt")) == NULL)
		return FALSE;

	point = NULL;
	face = NULL;
	line = NULL;
	side = NULL;

	while(!feof(fp)) {
		memset(buf, 0, sizeof(buf));
		if(fgets(buf, sizeof(buf), fp) == NULL)
			break;
		p = buf;
		row++;

		switch(getCommand(&p)) {
		case 'P':
			new_point = addPoint();
			new_point->no = getValue(&p);
			new_point->x = getValue(&p);
			new_point->y = getValue(&p);
			new_point->z = getValue(&p);
			break;
		case 'F':
			new_face = addFace();
			for(i = 0;; i++)
				if((new_face->p[i] = getValue(&p)) == INT_MIN) {
					new_face->n = i;
					break;
				}
			break;
		case 'L':
			new_line = addLine();
			for(i = 0;; i++)
				if((new_line->p[i] = getValue(&p)) == INT_MIN) {
					new_line->n = i;
					break;
				}
			break;
		}
	}

	fclose(fp);

	for(f = face; f->n >= 0; f++)
		for(i = 0; i < f->n; i++) {
			int p0 = f->p[i], p1 = f->p[(i + 1) % f->n];
			struct Side *new_side;

			if(p0 > p1)
				swap(&p0, &p1);

			if(findSide(p0, p1) == NULL) {
				new_side = addSide();
				new_side->p[0] = p0;
				new_side->p[1] = p1;
			}
		}

	return TRUE;
}

/*
	ASMt@Cۑ
*/
static int saveAsmFile(const char *path)
{
	int i, j;
	char base_name[MAX_PATH], dir_name[MAX_PATH], asm_path[MAX_PATH];
	FILE *fp;

	getFileBaseName(base_name, path);

	getDirName(dir_name, path);
	if(strcmp(dir_name, "") != 0)
		sprintf(asm_path, "%s%s.asm", dir_name, base_name);
	else
		sprintf(asm_path, "%s.asm", base_name);
	printf("%s\n%s\n", path, asm_path);

	if((fp = fopen(asm_path, "wt")) == NULL)
		return FALSE;

	fprintf(fp, "%s::\n", base_name);
	fprintf(fp, "\t.dw %d ; a\n", getRadius());
	fprintf(fp, "\n");
	fprintf(fp, "\t.db %d ; _̐\n", getPointCount());
	for(i = 0; i < getPointCount(); i++)
		fprintf(fp, "\t.dw %d, %d, %d ; _%d\n", point[i].x, point[i].y, point[i].z, i);
	fprintf(fp, "\n");
	fprintf(fp, "\t.db %d ; ʂ̐\n", getFaceCount());
	for(i = 0; i < getFaceCount(); i++) {
		fprintf(fp, "\t.dw %d * SIZE_VERTEX, %d * SIZE_VERTEX, %d * SIZE_VERTEX ; %d̕\\_\n", getPointIndex(face[i].p[0]), getPointIndex(face[i].p[1]), getPointIndex(face[i].p[2]), i);
		fprintf(fp, "\t.db %d ; %d̕ӂ̐\n", face[i].n, i);
		fprintf(fp, "\t.dw %s%d", base_name, (int)(findSide(face[i].p[0], face[i].p[1]) - side));
		for(j = 1; j < face[i].n; j++)
			fprintf(fp, ", %s%d", base_name, (int)(findSide(face[i].p[j], face[i].p[(j + 1) % face[i].n]) - side));
		fprintf(fp, " ; %d̕\n", i);
	}
	fprintf(fp, "\n");
	fprintf(fp, "\t.db %d ; ӂ̐\n", getSideCount());
	for(i = 0; i < getSideCount(); i++) {
		fprintf(fp, "%s%d:\n", base_name, i);
		fprintf(fp, "\t.db 0\n");
		fprintf(fp, "\t.dw %d * SIZE_VERTEX, %d * SIZE_VERTEX ; %d̗[̓_\n", getPointIndex(side[i].p[0]), getPointIndex(side[i].p[1]), i);
	}
	fprintf(fp, "\n");
	fprintf(fp, "\t.db %d ; ̐\n", getLineSum());
	for(i = 0; i < getLineSum(); i++)
		for(j = 0; j < line[i].n - 1; j++)
			fprintf(fp, "\t.dw %d * SIZE_VERTEX, %d * SIZE_VERTEX ; ̗[̓_\n", getPointIndex(line[i].p[j]), getPointIndex(line[i].p[j + 1]));

	fclose(fp);
	return TRUE;
}

/*
	ϊs
*/
static struct Point trans(const struct Point *p, const double *mat)
{
	double y, scale = 30.0;
	struct Point result;

	y = (p->x * mat[3] + p->y * mat[4] + p->z * mat[5] + mat[10]);
	result.y = y;

	if (y > .0) {
		result.x = (p->x * mat[0] + p->y * mat[1] + p->z * mat[2] + mat[9]) * scale / (y / 10.0 + 100.0);
		result.z = (p->x * mat[6] + p->y * mat[7] + p->z * mat[8] + mat[11]) * scale / (y / 10.0 + 100.0);
		return result;
	} else {
		return result;
	}
}

/*
	s̐ς߂
*/
static double *mmul(double *ans, double *mat0, double *mat1)
{
	double tmp[12];

	tmp[0] = mat0[0] * mat1[0] + mat0[1] * mat1[3] + mat0[2] * mat1[6];
	tmp[1] = mat0[0] * mat1[1] + mat0[1] * mat1[4] + mat0[2] * mat1[7];
	tmp[2] = mat0[0] * mat1[2] + mat0[1] * mat1[5] + mat0[2] * mat1[8];
	tmp[3] = mat0[3] * mat1[0] + mat0[4] * mat1[3] + mat0[5] * mat1[6];
	tmp[4] = mat0[3] * mat1[1] + mat0[4] * mat1[4] + mat0[5] * mat1[7];
	tmp[5] = mat0[3] * mat1[2] + mat0[4] * mat1[5] + mat0[5] * mat1[8];
	tmp[6] = mat0[6] * mat1[0] + mat0[7] * mat1[3] + mat0[8] * mat1[6];
	tmp[7] = mat0[6] * mat1[1] + mat0[7] * mat1[4] + mat0[8] * mat1[7];
	tmp[8] = mat0[6] * mat1[2] + mat0[7] * mat1[5] + mat0[8] * mat1[8];
	tmp[9] = mat0[9];
	tmp[10] = mat0[10];
	tmp[11] = mat0[11];

	memcpy(ans, tmp, sizeof(tmp));
	return ans;
}

/*
	XYʂŉ]
*/
static double *rotateXY(double *mat, double r)
{
	double m[] = {
		cos(r), -sin(r), 0.0,
		sin(r), cos(r), 0.0,
		0.0, 0.0, 1.0
	};
	return mmul(mat, mat, m);
}

/*
	YZʂŉ]
*/
static double *rotateYZ(double *mat, double r)
{
	double m[] = {
		1.0, 0.0, 0.0,
		0.0, cos(r), -sin(r),
		0.0, sin(r), cos(r)
	};
	return mmul(mat, mat, m);
}

/*
	ZXʂŉ]
*/
static double *rotateZX(double *mat, double r)
{
	double m[] = {
		cos(r), 0.0, -sin(r),
		0.0, 0.1, 0.0,
		sin(r), 0.0, cos(r)
	};
	return mmul(mat, mat, m);
}

/*
	f`悷
*/
static void drawModel(SDL_Surface *screen, SDL_Rect clip, const double *mat)
{
	int i, cx = clip.w / 2, cz = clip.h / 2;
	struct Face *f;
	struct Line *l;

	if(face != NULL) {
		for(f = face; f->n >= 0; f++)
			for(i = 0; i < f->n; i++) {
				struct Point *p0 = findPoint(f->p[i]), *p1 = findPoint(f->p[(i + 1) % f->n]), *p2 = findPoint(f->p[(i + 2) % f->n]);

				if(p0 != NULL && p1 != NULL && p2 != NULL) {
					struct Point q0 = trans(p0, mat), q1 = trans(p1, mat), q2 = trans(p2, mat);

					if(q0.y > 0 && q1.y > 0) {
						int ax = q0.x - q1.x, az = q0.z - q1.z, bx = q2.x - q1.x, bz = q2.z - q1.z;

						if(ax * bz - az * bx >= 0) {
							drawNumber(screen, &clip, cx + q0.x, cz - q0.z, p0->no);
							drawNumber(screen, &clip, cx + q1.x, cz - q1.z, p1->no);
							drawLine(screen, &clip, cx + q0.x, cz - q0.z, cx + q1.x, cz - q1.z, white);
						}
					}
				}
			}
	}

	if(line != NULL) {
		for(l = line; l->n >= 0; l++)
			for(i = 0; i < l->n; i++) {
				struct Point *p0 = findPoint(l->p[i]), *p1 = findPoint(l->p[(i + 1) % l->n]);

				if(p0 != NULL && p1 != NULL) {
					struct Point q0 = trans(p0, mat), q1 = trans(p1, mat);

					if(q0.y > 0 && q1.y > 0) {
						drawNumber(screen, &clip, cx + q0.x, cz - q0.z, p0->no);
						drawNumber(screen, &clip, cx + q1.x, cz - q1.z, p1->no);
						drawLine(screen, &clip, cx + q0.x, cz - q0.z, cx + q1.x, cz - q1.z, white);
					}
				}
			}
	}
}

/*
	C
*/
int main(int argc, char *argv[])
{
	double mat1[] = {
		1.0, 0.0, 0.0,
		0.0, 1.0, 0.0,
		0.0, 0.0, 1.0,
		0.0, 0.0, 0.0
	};
	double mat_xy[] = {
		0.0, -1.0, 0.0,
		0.0, 0.0, 1.0,
		1.0, 0.0, 0.0,
		0.0, 0.0, 0.0
	};
	double mat_yz[] = {
		0.0, 1.0, 0.0,
		0.0, 0.0, 1.0,
		1.0, 0.0, 0.0,
		0.0, 0.0, 0.0
	};
	double mat_zx[] = {
		0.0, 0.0, 1.0,
		1.0, 0.0, 0.0,
		0.0, 1.0, 0.0,
		0.0, 0.0, 0.0
	};
	const SDL_Rect rect_xy = { 0, 0, 320, 240 }, rect_yz = { 0, 240, 320, 240 }, rect_zx = { 320, 0, 320, 240 }, rect_roll = { 320, 240, 320, 240 };
	SDL_Window *window;
	SDL_Surface *screen;
	struct stat old_stat;
	double mat[12], r = 0.0;
	char model_file[MAX_PATH];

	/* SDL */
	SDL_SetMainReady();
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_EVENTS) < 0) {
		fprintf(stderr, "SDL_Init fail. %s\n", SDL_GetError());
		exit(1);
	}
	if((window = SDL_CreateWindow("3D test", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, 640, 480, 0)) == NULL) {
		fprintf(stderr, "SDL_CreateWindow fail. %s\n", SDL_GetError());
		exit(1);
	}
	if((screen = SDL_GetWindowSurface(window)) == NULL) {
		fprintf(stderr, "SDL_GetWindowSurface fail. %s\n", SDL_GetError());
		exit(1);
	}

	number[0] = SDL_CreateRGBSurfaceFromXpm(zero_xpm, NULL);
	number[1] = SDL_CreateRGBSurfaceFromXpm(one_xpm, NULL);
	number[2] = SDL_CreateRGBSurfaceFromXpm(two_xpm, NULL);
	number[3] = SDL_CreateRGBSurfaceFromXpm(three_xpm, NULL);
	number[4] = SDL_CreateRGBSurfaceFromXpm(four_xpm, NULL);
	number[5] = SDL_CreateRGBSurfaceFromXpm(five_xpm, NULL);
	number[6] = SDL_CreateRGBSurfaceFromXpm(six_xpm, NULL);
	number[7] = SDL_CreateRGBSurfaceFromXpm(seven_xpm, NULL);
	number[8] = SDL_CreateRGBSurfaceFromXpm(eight_xpm, NULL);
	number[9] = SDL_CreateRGBSurfaceFromXpm(nine_xpm, NULL);

	/*  */
	if(argc >= 2)
		strcpy(model_file, argv[1]);
	else
		strcpy(model_file, "");

	memset(&old_stat, 0, sizeof(old_stat));
	memcpy(mat, mat1, sizeof(mat));

	for(;;) {
		SDL_Event e;
		struct stat cur_stat;

		/* ft@Cςēǂݍ݂ */
		if(strcmp(model_file, "") != 0)
			if(stat(model_file, &cur_stat) >= 0)
				if(cur_stat.st_mtime != old_stat.st_mtime) {
					if(readModelFile(model_file)) {
						saveAsmFile(model_file);
						mat1[10] = getRadius() + 1;
					}
					old_stat = cur_stat;
				}

		memcpy(mat, mat1, sizeof(mat));
		rotateXY(mat, r);
		SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
		drawModel(screen, rect_roll, mat);

		memcpy(mat, mat1, sizeof(mat));
		rotateYZ(mat, r);
		drawModel(screen, rect_xy, mat);

		r += .1;

		memcpy(mat, mat1, sizeof(mat));
		rotateXY(mat, 2.0 * M_PI / 4.0);
		drawModel(screen, rect_yz, mat);

		memcpy(mat, mat1, sizeof(mat));
		rotateYZ(mat, 2.0 * M_PI / 4.0);
		drawModel(screen, rect_zx, mat);

#if 0
		/* TEST */
		drawCircle(screen, 320, 240, 32, white);
		/* TEST */
#endif

		while(SDL_PollEvent(&e)) {
			if(e.type == SDL_QUIT) {
				SDL_Quit();
				return 0;
			} else if(/*e.type == SDL_MOUSEBUTTONDOWN ||*/ e.type == SDL_KEYDOWN) {
				SDL_Quit();
				return 0;
			} else if(e.type == SDL_DROPFILE) {
				strcpy(model_file, e.drop.file);
			} else if(e.type == SDL_MOUSEWHEEL) {
				/*
				double d;

				if(mat1[10] > getRadius() / 2)
					d = fabs(mat1[10]) / 2;
				else
					d = getRadius() / 2;

				if(e.wheel.y > 0)
					mat1[10] -= d;
				else
					mat1[10] += d;
				*/
			}
		}
		SDL_UpdateWindowSurface(window);
		SDL_Delay(100);
	}

	return 0;
}

/* eof */